@@ -1,4 +1,4 @@
-
+#!/usr/bin/python
 # Twisted, the Framework of Your Internet
 # Copyright (C) 2001 Matthew W. Lefkowitz
 # 
@@ -14,32 +14,24 @@
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
 
-from twisted.protocols.protocol import Protocol, Factory
-from twisted.internet import udp
+from twisted.internet.protocol import Protocol, Factory
+from twisted.internet import reactor
 
 ### Protocol Implementation
 
 # This is just about the simplest possible protocol
-
 class Echo(Protocol):
     def dataReceived(self, data):
-        "As soon as any data is received, write it back."
+        """As soon as any data is received, write it back."""
         self.transport.write(data)
 
-### Persistent Application Builder
-
-# This builds a .tap file
+def main():
+    f = Factory()
+    f.protocol = Echo
+    reactor.listenTCP(8000, f)
+    reactor.run()
 
 if __name__ == '__main__':
-    # Since this is persistent, it's important to get the module naming right
-    # (If we just used Echo, then it would be __main__.Echo when it attempted
-    # to unpickle)
-    import echoserv
-    from twisted.internet.app import Application
-    factory = Factory()
-    factory.protocol = echoserv.Echo
-    app = Application("echo")
-    app.listenTCP(8000,factory)
-    app.listenUDP(8000, factory)
-    app.save("start")
+    main()
