@@ -1,4 +1,5 @@
-
+# -*- Python -*-
+# $Id: usage.py,v 1.56 2003/10/29 17:28:03 tv Exp $
 # Twisted, the Framework of Your Internet
 # Copyright (C) 2001 Matthew W. Lefkowitz
 #
@@ -17,51 +18,12 @@
 
 """
 twisted.python.usage is a module for parsing/handling the
-command line of your program. You use it by subclassing
-Options with certain methods/attributes defined. Here's
-an example::
+command line of your program.
 
-    from twisted.python import usage
-    import sys
-    class MyOptions(usage.Options):
-        optFlags = [["hello", "h"], ["goodbye", "g"]]
-        optParameters = [["message", "m", "friend!"]]
-        def __init__(self):
-            self.debug = 0
-        def opt_debug(self, opt):
-            if opt == "yes" or opt == "y" or opt == "1":
-                self.debug = 1
-            elif opt == "no" or opt == "n" or opt == "0":
-                self.debug = 0
-            else:
-                print "Unknown value for debug, setting to 0"
-                self.debug = 0
-        opt_d = opt_debug # a single-char alias for --debug
-    try:
-        config = MyOptions()
-        config.parseOptions()
-    except usage.UsageError, ue:
-        print "%s: %s" % (sys.argv[0], ue)
-    if config.hello:
-        if config.debug: print "printing hello"
-        print "hello", config.message #defaults to "friend!"
-    if config.goodbye:
-        if config.debug: print "printing goodbye"
-        print "goodbye", config.message
-
-#EOF
-
-As you can see, you define optFlags as a list of paramaters (with
-both long and short names) that are either on or off.  optStrings
-are paramaters with string values, with their default value as
-the third parameter in the list.  If you want to handle your own
-options, define a method named opt_paramname that takes (self,
-option) as paramaters. option will be whatever immediately follows
-the paramater on the command line. A few example command lines
-that will work:
-
-# XXX - Where'd the examples go?
-""" # "
+For information on how to use it, see
+U{http://twistedmatrix.com/documents/howto/options}, or doc/howto/options.html
+in your Twisted directory.
+"""
 
 # System Imports
 import string
@@ -74,20 +36,86 @@ from os import path
 # Sibling Imports
 import reflect
 import text
-
+import util
+import log
 
 class UsageError(Exception):
     pass
 
 error = UsageError
 
-class Options:
+class Options(dict):
     """
-    A class which can be subclassed to provide command-line options
-    to your program. See twisted.usage.__doc__ for for details.
+    An option list parser class
+
+    C{optFlags} and C{optParameters} are lists of available parameters
+    which your program can handle. The difference between the two
+    is the 'flags' have an on(1) or off(0) state (off by default)
+    whereas 'parameters' have an assigned value, with an optional
+    default. (Compare '--verbose' and '--verbosity=2')
+
+    optFlags is assigned a list of lists. Each list represents
+    a flag parameter, as so::
+
+    |    optFlags = [['verbose', 'v', 'Makes it tell you what it doing.'],
+    |                ['quiet', 'q', 'Be vewy vewy quiet.']]
+
+    As you can see, the first item is the long option name
+    (prefixed with '--' on the command line), followed by the
+    short option name (prefixed with '-'), and the description.
+    The description is used for the built-in handling of the
+    --help switch, which prints a usage summary.
+
+    C{optParameters} is much the same, except the list also contains
+    a default value::
+
+    | optParameters = [['outfile', 'O', 'outfile.log', 'Description...']]
+
+    subCommands is a list of 4-tuples of (command name, command shortcut,
+    parser class, documentation).  If the first non-option argument found is
+    one of the given command names, an instance of the given parser class is
+    instantiated and given the remainder of the arguments to parse and
+    self.opts[command] is set to the command name.  For example::
+
+    | subCommands = [
+    |      ['inquisition', 'inquest', InquisitionOptions, 'Perform an inquisition'],
+    |      ['holyquest', 'quest', HolyQuestOptions, 'Embark upon a holy quest']
+    |  ]
+
+    In this case, C{"<program> holyquest --horseback --for-grail"} will cause
+    C{HolyQuestOptions} to be instantiated and asked to parse
+    C{['--horseback', '--for-grail']}.  Currently, only the first sub-command
+    is parsed, and all options following it are passed to its parser.  If a
+    subcommand is found, the subCommand attribute is set to its name and the
+    subOptions attribute is set to the Option instance that parses the
+    remaining options. If a subcommand is not given to parseOptions,
+    the subCommand attribute will be None. You can also mark one of
+    the subCommands to be the default.
+
+    | defaultSubCommand = 'holyquest'
+
+    In this case, the subCommand attribute will never be None, and
+    the subOptions attribute will always be set.
+
+    If you want to handle your own options, define a method named
+    C{opt_paramname} that takes C{(self, option)} as arguments. C{option}
+    will be whatever immediately follows the parameter on the
+    command line. Options fully supports the mapping interface, so you
+    can do things like C{'self["option"] = val'} in these methods.
+
+    Advanced functionality is covered in the howto documentation,
+    available at U{http://twistedmatrix.com/documents/howto/options}, or
+    doc/howto/options.html in you Twisted directory.
     """
 
+    subCommand = None
+    defaultSubCommand = None
+    parent = None
     def __init__(self):
+        super(Options, self).__init__()
+
+        self.opts = self
+
         # These are strings/lists we will pass to getopt
         self.longOpt = []
         self.shortOpt = ''
@@ -95,26 +123,39 @@ class Options:
         self.synonyms = {}
         self.__dispatch = {}
 
+
         collectors = [
             self._gather_flags,
             self._gather_parameters,
             self._gather_handlers,
             ]
 
-        for (longOpt, shortOpt, docs, settings, synonyms, dispatch)\
-            in map( lambda c: c(), collectors):
-
+        for c in collectors:
+            (longOpt, shortOpt, docs, settings, synonyms, dispatch) = c()
             self.longOpt.extend(longOpt)
             self.shortOpt = self.shortOpt + shortOpt
             self.docs.update(docs)
-            self.__dict__.update(settings)
+
+            self.opts.update(settings)
+
             self.synonyms.update(synonyms)
             self.__dispatch.update(dispatch)
 
+    def __hash__(self):
+        # This is required because dicts aren't hashable by default
+        # (They define __cmp__ but no __hash__)
+        return id(self)
+
     def opt_help(self):
+        """Display this help and exit."""
         print self.__str__()
         sys.exit(0)
 
+    def opt_version(self):
+        from twisted import copyright
+        print "Twisted version:", copyright.version
+        sys.exit(0)
+
     #opt_h = opt_help # this conflicted with existing 'host' options.
 
     def parseOptions(self, options=None):
@@ -128,12 +169,8 @@ class Options:
             opts, args = getopt.getopt(options,
                                        self.shortOpt, self.longOpt)
         except getopt.error, e:
-            raise UsageError, e
+            raise UsageError, str(e).capitalize()
 
-        try:
-            apply(self.parseArgs,args)
-        except TypeError:
-            raise error, "wrong number of arguments."
 
         for opt, arg in opts:
             if opt[1] == '-':
@@ -141,22 +178,43 @@ class Options:
             else:
                 opt = opt[1:]
 
-            if not self.synonyms.has_key(opt):
-                raise UsageError, "No such option '%s'" % (opt,)
+            optMangled = opt
+            if not self.synonyms.has_key(optMangled):
+                optMangled = string.replace(opt, "-", "_")
+                if not self.synonyms.has_key(optMangled):
+                    raise UsageError, "No such option '%s'" % (opt,)
 
-            opt = self.synonyms[opt]
-            self.__dispatch[opt](opt, arg)
+            optMangled = self.synonyms[optMangled]
+            self.__dispatch[optMangled](optMangled, arg)
+
+        if (getattr(self, 'subCommands', None)
+            and (args or self.defaultSubCommand is not None)):
+            if not args:
+                args = [self.defaultSubCommand]
+            sub, rest = args[0], args[1:]
+            for (cmd, short, parser, doc) in self.subCommands:
+                if sub == cmd or sub == short:
+                    self.subCommand = cmd
+                    self.subOptions = parser()
+                    self.subOptions.parent = self
+                    self.subOptions.parseOptions(rest)
+                    break
+            else:
+                raise UsageError("Unknown command: %s" % sub)
+        else:
+            try:
+                self.parseArgs(*args)
+            except TypeError:
+                raise UsageError("Wrong number of arguments.")
 
         self.postOptions()
 
-
     def postOptions(self):
         """I am called after the options are parsed.
 
         Override this method in your subclass to do something after
-        the options have been parsed and assigned.
-
-        XXX: Like what?
+        the options have been parsed and assigned, like validate that
+        all options are sane.
         """
         pass
 
@@ -180,13 +238,15 @@ class Options:
         if value not in ('', None):
             raise UsageError, ("Flag '%s' takes no argument."
                                " Not even \"%s\"." % (flagName, value))
-        setattr(self, flagName, 1)
+
+        self.opts[flagName] = 1
 
     def _generic_parameter(self, parameterName, value):
-        if value in ('', None):
+        if value is None:
             raise UsageError, ("Parameter '%s' requires an argument."
                                % (parameterName,))
-        setattr(self, parameterName, value)
+
+        self.opts[parameterName] = value
 
     def _gather_flags(self):
         """Gather up boolean (flag) options.
@@ -199,7 +259,7 @@ class Options:
         reflect.accumulateClassList(self.__class__, 'optFlags', flags)
 
         for flag in flags:
-            long, short, doc = padTo(3, flag)
+            long, short, doc = util.padTo(3, flag)
             if not long:
                 raise ValueError, "A flag cannot be without a name."
 
@@ -222,21 +282,19 @@ class Options:
         docs, settings, synonyms, dispatch = {}, {}, {}, {}
 
         parameters = []
-        # We have to keep calling these "optStrings" because this code is
-        # used in the IPC10 paper, which makes it written in stone...
+
         reflect.accumulateClassList(self.__class__, 'optStrings',
                                     parameters)
+        if parameters:
+            log.msg("Warning: Options.optStrings is deprecated, please use optParameters instead.")
 
-        # But since "strings" is a very poor description (yes, getopt
-        # does happen to return the values as strings, but that's
-        # irrelevant), provide another name that makes more sense.
         reflect.accumulateClassList(self.__class__, 'optParameters',
                                     parameters)
 
         synonyms = {}
 
         for parameter in parameters:
-            long, short, default, doc = padTo(4, parameter)
+            long, short, default, doc = util.padTo(4, parameter)
             if not long:
                 raise ValueError, "A parameter cannot be without a name."
 
@@ -264,36 +322,31 @@ class Options:
 
         for name in dct.keys():
             method = getattr(self, 'opt_'+name)
-            reqArgs = method.im_func.func_code.co_argcount
-            if reqArgs > 2:
-                raise error, 'invalid Option function for %s' % name
-            if reqArgs == 2:
-                # argName = method.im_func.func_code.co_varnames[1]
-                takesArg = 1
-            else:
-                takesArg = 0
 
+            takesArg = not flagFunction(method, name)
+
+            prettyName = string.replace(name, '_', '-')
             doc = getattr(method, '__doc__', None)
             if doc:
                 ## Only use the first line.
                 #docs[name] = string.split(doc, '\n')[0]
-                docs[name] = doc
+                docs[prettyName] = doc
             else:
-                docs[name] = None
+                docs[prettyName] = self.docs.get(prettyName)
 
-            synonyms[name] = name
+            synonyms[prettyName] = prettyName
 
             # A little slight-of-hand here makes dispatching much easier
             # in parseOptions, as it makes all option-methods have the
             # same signature.
             if takesArg:
-                fn = lambda self, name, value, m=method: m(value)
+                fn = lambda name, value, m=method: m(value)
             else:
                 # XXX: This won't raise a TypeError if it's called
                 # with a value when it shouldn't be.
-                fn = lambda self, name, value=None, m=method: m()
+                fn = lambda name, value=None, m=method: m()
 
-            dispatch[name] = new.instancemethod(fn, self, self.__class__)
+            dispatch[prettyName] = fn
 
             if len(name) == 1:
                 shortOpt = shortOpt + name
@@ -301,8 +354,8 @@ class Options:
                     shortOpt = shortOpt + ':'
             else:
                 if takesArg:
-                    name = name + '='
-                longOpt.append(name)
+                    prettyName = prettyName + '='
+                longOpt.append(prettyName)
 
         reverse_dct = {}
         # Map synonyms
@@ -327,9 +380,49 @@ class Options:
 
 
     def __str__(self, width=None):
+        return self.getSynopsis() + '\n' + self.getUsage(width=None)
+
+    def getSynopsis(self):
+        default = "%s%s" % (path.basename(sys.argv[0]),
+                            (self.longOpt and " [options]") or '')
+        if self.parent is None:
+            default = "Usage: %s%s" % (path.basename(sys.argv[0]),
+                                       (self.longOpt and " [options]") or '')
+        else:
+            default = '%s' % ((self.longOpt and "[options]") or '')
+        synopsis = getattr(self, "synopsis", default)
+
+        synopsis = synopsis.rstrip()
+
+        if self.parent is not None:
+            synopsis = ' '.join((self.parent.getSynopsis(), self.parent.subCommand, synopsis))
+
+        return synopsis
+
+    def getUsage(self, width=None):
+        #If subOptions exists by now, then there was probably an error while
+        #parsing its options.
+        if hasattr(self, 'subOptions'):
+            return self.subOptions.getUsage(width=width)
+
         if not width:
             width = int(os.environ.get('COLUMNS', '80'))
 
+        if hasattr(self, 'subCommands'):
+            cmdDicts = []
+            for (cmd, short, parser, desc) in self.subCommands:
+                cmdDicts.append(
+                    {'long': cmd,
+                     'short': short,
+                     'doc': desc,
+                     'optType': 'command',
+                     'default': None
+                    })
+            chunks = docMakeChunks(cmdDicts, width)
+            commands = 'Commands:\n' + ''.join(chunks)
+        else:
+            commands = ''
+
         longToShort = {}
         for key, value in self.synonyms.items():
             longname = value
@@ -354,17 +447,9 @@ class Options:
                  'short': longToShort[opt],
                  'doc': self.docs[opt],
                  'optType': optType,
-                 'default': getattr(self, opt, None)
+                 'default': self.opts.get(opt, None)
                  })
 
-        synopsis = getattr(self, "synopsis",
-                           "Usage: %s%s"
-                           % (path.basename(sys.argv[0]),
-                              (optDicts and " [options]") or ''))
-
-        if not synopsis[-len('\n'):] == '\n':
-            synopsis = synopsis + '\n'
-
         if not (getattr(self, "longdesc", None) is None):
             longdesc = self.longdesc
         else:
@@ -376,7 +461,8 @@ class Options:
 
         if longdesc:
             longdesc = ('\n' +
-                        string.join(text.wordWrap(longdesc, width), '\n'))
+                        string.join(text.wordWrap(longdesc, width), '\n').strip()
+                        + '\n')
 
         if optDicts:
             chunks = docMakeChunks(optDicts, width)
@@ -384,7 +470,7 @@ class Options:
         else:
             s = "Options: None\n"
 
-        return synopsis + s + longdesc
+        return s + longdesc + commands
 
     #def __repr__(self):
     #    XXX: It'd be cool if we could return a succinct representation
@@ -395,7 +481,7 @@ def docMakeChunks(optList, width=80):
     """Makes doc chunks for option declarations.
 
     Takes a list of dictionaries, each of which may have one or more
-    of the keys "long", "short", "doc", "default", "optType".
+    of the keys 'long', 'short', 'doc', 'default', 'optType'.
 
     Returns a list of strings.
     The strings may be multiple lines,
@@ -408,7 +494,7 @@ def docMakeChunks(optList, width=80):
     for opt in optList:
         optLen = len(opt.get('long', ''))
         if optLen:
-            if opt.get('optType', None) != "flag":
+            if opt.get('optType', None) == "parameter":
                 # these take up an extra character
                 optLen = optLen + 1
             maxOptLen = max(optLen, maxOptLen)
@@ -422,7 +508,15 @@ def docMakeChunks(optList, width=80):
     colFiller1 = " " * colWidth1
 
     optChunks = []
+    seen = {}
     for opt in optList:
+        if seen.has_key(opt.get('short', None)) \
+           or seen.has_key(opt.get('long', None)):
+            continue
+        for x in opt.get('short', None), opt.get('long', None):
+            if x is not None:
+                seen[x]=1
+
         optLines = []
         comma = " "
         if opt.get('short', None):
@@ -432,7 +526,7 @@ def docMakeChunks(optList, width=80):
 
         if opt.get('long', None):
             long = opt['long']
-            if opt.get("optType", None) != "flag":
+            if opt.get("optType", None) == "parameter":
                 long = long + '='
 
             long = "%-*s" % (maxOptLen, long)
@@ -441,14 +535,17 @@ def docMakeChunks(optList, width=80):
         else:
             long = " " * (maxOptLen + len('--'))
 
-        column1 = "  %2s%c --%s  " % (short, comma, long)
+        if opt.get('optType', None) == 'command':
+            column1 = '    %s      ' % long
+        else:
+            column1 = "  %2s%c --%s  " % (short, comma, long)
 
         if opt.get('doc', ''):
-            doc = opt['doc']
+            doc = opt['doc'].strip()
         else:
             doc = ''
 
-        if (opt.get("optType", None) != "flag") \
+        if (opt.get("optType", None) == "parameter") \
            and not (opt.get('default', None) is None):
             doc = "%s [default: %s]" % (doc, opt['default'])
 
@@ -466,24 +563,11 @@ def docMakeChunks(optList, width=80):
 
     return optChunks
 
-
-def padTo(n, seq, default=None):
-    """Pads a sequence out to n elements,
-
-    filling in with a default value if it is not long enough.
-
-    If the input sequence is longer than n, raises ValueError.
-
-    Details, details:
-    This returns a new list; it does not extend the original sequence.
-    The new list contains the values of the original sequence, not copies.
-    """
-
-    if len(seq) > n:
-        raise ValueError, "%d elements is more than %d." % (len(seq), n)
-
-    blank = [default] * n
-
-    blank[:len(seq)] = list(seq)
-
-    return blank
+def flagFunction(method, name = None):
+    reqArgs = method.im_func.func_code.co_argcount
+    if reqArgs > 2:
+        raise UsageError('Invalid Option function for %s' % (name or method.func_name))
+    if reqArgs == 2:
+        # argName = method.im_func.func_code.co_varnames[1]
+        return 0
+    return 1
