@@ -16,5 +16,10 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 """
+DEPRECATED.
+
 Twisted Spread UI: UI utilities for various toolkits connecting to PB.
 """
+
+import warnings
+warnings.warn("twisted.spread.ui is deprecated. Please do not use.", DeprecationWarning)
